/**********************************************************************
* MOUSE5.C - Draw various hardware and software text mode mouse       *
*   cursors on the screen.                                            *
*                                                                     *
* To compile: "cl mouse5.c"                                           *
**********************************************************************/
#include <dos.h>    /* int86(), union REGS                           */
#include <stdlib.h> /* exit(), EXIT_FAILURE                          */
#include <conio.h>  /* kbhit(), getch()                              */
#include <graph.h>  /* _setvideomode(), _DEFAULTMODE                 */

#pragma comment(lib, "graphics.lib") /* Include library-search record*/

/*-------- Bit values for screen attribute bytes --------------------*/
#define BLUE_FORE     1
#define GREEN_FORE    2
#define RED_FORE      4
#define INTENSITY     8
#define BLUE_BACK     16
#define GREEN_BACK    32
#define RED_BACK      64
#define BLINK         128

/*-------- Mask values for calls to SetSoftwareCursor() ------------*/
#define RETAIN        0xff   /* Retain entire screen char or attr   */
#define DISCARD       0x00   /* Discard screen char OR no mask char */
#define INVERT        0x77   /* Invert attributes of screen char    */

/*-------- Miscellaneous defines -----------------------------------*/
#define UP_ARROW      0x18   /* ASCII code for the up arrow char    */
#define NONEPRESSED   0      /* Our signal that no buttons pressed  */
#define LEFTPRESSED   1      /* Our signal that left button pressed */
#define RIGHTPRESSED  2      /* Our signal that right button pressed*/
#define BOTHPRESSED   3      /* Our signal that both buttons pressed*/

/*-------- Function Prototypes -------------------------------------*/
void InitializeMouse(void);
void ShowMouseCursor(void);
void HideMouseCursor(void);
void SetHardwareCursor(int Top, int Bottom);
void SetSoftwareCursor(int scra, int scrc, int cura, int curc );
int GetPressed(void);

union REGS in_regs, out_regs;     /* Global register structures     */

/*********************************************************************
* InitializeMouse - Returns on success, exits to DOS on failure      *
*********************************************************************/
void InitializeMouse(void)
    {
    in_regs.x.ax = 0x00;	/* select mouse service 0    */
    int86( 0x33, &in_regs, &out_regs );
    if ( out_regs.x.ax )
	    puts( "Mouse initialized successfully" );
    else
        {
        puts( "ERROR-can't initialize mouse" );
        exit( EXIT_FAILURE );
        }
    }

/*********************************************************************
* ShowMouseCursor - Turns on mouse cursor                            *
*********************************************************************/
void ShowMouseCursor(void)
    {
    in_regs.x.ax = 0x01;
    int86( 0x33, &in_regs, &out_regs );
    }

/*********************************************************************
* HideMouseCursor - Turns off mouse cursor                           *
*********************************************************************/
void HideMouseCursor(void)
    {
    in_regs.x.ax = 0x02;          /* Select mouse service 2         */
    int86( 0x33, &in_regs, &out_regs );  /* Call mouse driver       */
    }

/*********************************************************************
* SetHardwareCursor - Sets new hardware cursor using TopLine and     *
*   BottomLine arguments. See IMC article for details.               *
*********************************************************************/
void SetHardwareCursor(int TopLine, int BottomLine)
    {
    HideMouseCursor();            /* Turn off cursor before changing*/
    in_regs.x.ax = 0x0a;	       /* Select mouse service 10        */
    in_regs.x.bx = 0x01;          /* Select hardware text cursor    */
    in_regs.x.cx = TopLine;       /* Load first scan line into CX   */
    in_regs.x.dx = BottomLine;    /* Load last scan line into DX    */
    int86( 0x33, &in_regs, &out_regs );  /* Call mouse driver       */
    ShowMouseCursor();            /* Turn new cursor on             */
    }

/*********************************************************************
* SetSoftwareCursor - Uses the four mask arguments to specify the    *
*   appearance of the software cursor. See IMC article for details.  *
*********************************************************************/
void SetSoftwareCursor(int screen_mask_attr, int screen_mask_char,
                       int cursor_mask_attr, int cursor_mask_char )
    {
    HideMouseCursor();            /* Turn off cursor before changing*/
    in_regs.x.ax = 0x0a;	       /* Select mouse service 10        */
    in_regs.x.bx = 0x00;          /* Select software text cursor    */
    in_regs.h.ch = screen_mask_attr;
    in_regs.h.cl = screen_mask_char;
    in_regs.h.dh = cursor_mask_attr;
    in_regs.h.dl = cursor_mask_char;
    int86( 0x33, &in_regs, &out_regs );  /* Call mouse driver       */
    ShowMouseCursor();            /* Turn new cursor on             */
    }

/*********************************************************************
* GetPressed - Checks to see if any buttons have been pressed        *
*********************************************************************/
int GetPressed(void)
    {
    int rval;                     /* holds return code              */

    in_regs.x.ax = 0x05;          /* Select mouse function 5        */
    in_regs.x.bx = 0;             /* Inquire about left button      */
    int86( 0x33, &in_regs, &out_regs );  /* Call mouse driver       */
    rval = (out_regs.x.bx) ? LEFTPRESSED : 0;  /* Set bit           */

    in_regs.x.ax = 0x05;          /* Select mouse function 5        */
    in_regs.x.bx = 1;             /* Inquire about right button     */
    int86( 0x33, &in_regs, &out_regs );  /* Call mouse driver       */
    rval += (out_regs.x.bx) ? RIGHTPRESSED : 0;  /* Set bit         */
    return( rval );
    }

/*********************************************************************
* main - Initializes screen and mouse. Draws software mouse cursor   *
*   and waits for keystroke. After keystroke, changes to a hardware  *
*   mouse cursor and waits for another keystroke. Finally, changes   *
*   hardware cursor back to the standard DOS underscore and exits.   *
*********************************************************************/
void main( void )
    {
    int pressed;

    _setvideomode( _DEFAULTMODE );
    InitializeMouse();            /* Initialize mouse driver        */
    ShowMouseCursor();            /* Display default mouse cursor   */

    puts( "Press any key to quit ..." );

    while ( !kbhit() )            /* Loop until user presses a key  */
        {
        pressed = GetPressed();   /* Were any keys pressed?         */
        switch(pressed)
            {
            case LEFTPRESSED  :   /* Show black/white up arrow      */
                SetSoftwareCursor( DISCARD, DISCARD, 
                    BLUE_FORE + GREEN_FORE + RED_FORE, UP_ARROW );
                break;
            case RIGHTPRESSED :   /* Show reverse video block       */
                SetSoftwareCursor( RETAIN, RETAIN, INVERT, DISCARD );
                break;
            case BOTHPRESSED  :   /* Show square hardware cursor    */
                SetHardwareCursor(4, 7);
                break;
            case NONEPRESSED  :
                break;
            }
        }

    HideMouseCursor();            /* Turn off mouse cursor          */
    SetHardwareCursor(6, 7);      /* Reset std hardware text cursor */
    }


